package uir;

/**
 * Tda implementuje pevody mezi slem a slem reprezentovan eskmi slovy<br>
 * <br>
 * Poznmka: Pracuje s rozsahem od 0 do 1 000 000 000.
 * @author Martin Sloup, A04372<br>msloup@students.zcu.cz<br><br>Zpadoesk Univerzita<br>Fakulta Aplikovanch Vd
 */
public class CzechWordsNumber extends Number implements Comparable {
	private static final long serialVersionUID = 2399518449430616263L;
	private String stringValue;
	private int intValue;

	/**
	 * Konstruktor tdy s pouitm parametru reprezentovanou eskmi slovy
	 * @param s 		hodnota reprezentovan eskmi slovy
	 * @throws NumberFormatException
	 */
	public CzechWordsNumber(String s) throws NumberFormatException {
		stringValue = s;
		intValue = parseInt(s);
	}

	/**
	 * Konstruktor tdy s pouitm parametru reprezentovanou v destkov soustav
	 * @param value		hodnota v destkov soustav
	 * @throws NumberFormatException
	 */
	public CzechWordsNumber(int value) throws NumberFormatException {
		intValue = value;
		stringValue = toString(value);
	}

	/**
	 * Konstruktor tdy s pouitm parametru reprezentovanou v destkov soustav
	 * @param value 			hodnota v destkov soustav
	 * @param useChequeFormat 	pout formt vstupu na sloenky
	 * @throws NumberFormatException
	 */
	public CzechWordsNumber(int value, boolean useChequeFormat) throws NumberFormatException {
		intValue = value;
		stringValue = toString(value, useChequeFormat);
	}

	/**
	 * Vrac slo reprezentovan eskmi slovy
	 * @return slo reprezentovan eskmi slovy
	 */
	public String toString() {
		return stringValue;
	}

	/**
	 * Vrac hodnotu typu int
	 * @return hodnota typu int
	 */
	public int intValue() {
		return intValue;
	}

	/**
	 * Vrac hodnotu typu short
	 * @return hodnota typu short
	 */
	public short shortValue() {
		return (short) intValue;
	}

	/**
	 * Vrac hodnotu typu long
	 * @return hodnota typu long
	 */
	public long longValue() {
		return (long) intValue;
	}

	/**
	 * Vrac hodnotu typu double
	 * @return hodnota typu double
	 */
	public double doubleValue() {
		return (double) intValue;
	}

	/**
	 * Vrac hodnotu typu float
	 * @return hodnota typu float
	 */
	public float floatValue() {
		return (float) intValue;
	}

	/**
	 * Vrac hodnotu typu byte
	 * @return hodnota typu byte
	 */
	public byte byteValue() {
		return (byte) intValue;
	}

	/**
	 * Porovnv s objektem podtypu typu Number
	 * @param arg0 objekt podtypu typu Number
	 * @return -1, pokud je arg0 vt; 0, pokud jsou stejn velk; 1, pokud je arg0 men
	 * @throws ClassCastException
	 */
	public int compareTo(Object arg0) throws ClassCastException {
		if ((arg0 instanceof Double) || (arg0 instanceof Float)) {
			double value = ((Number)arg0).doubleValue();
			if (this.doubleValue() == value) {
				return 0;
			} else {
				return (this.doubleValue() > value) ? 1 : -1;
			}
		}
		if (arg0 instanceof Number)
			return (int) (this.longValue() - ((Number)arg0).longValue());


		throw new ClassCastException("Objekt musi podtyp typu Number.");
	}

	/**
	 * Porovnv hodnotu objektu s hodnotou typu int
	 * @param arg0 hodnota typu int, kter bude porovnna
	 * @return -1, pokud je arg0 vt; 0, pokud jsou stejn velk; 1, pokud je arg0 men
	 */
	public int compareTo(int arg0) {
		return this.intValue() - ((Number)arg0).intValue();
	}

	/**
	 * Porovnva zda zadan objekt obsahuje stejnou hodnotu (podtyp typu Number), ppadn zda je objekt toton
	 * @param obj objekt kter bude porovnvn
	 * @return true, pokud maj stejn ou hodnotu, ppadn jsou toton, jinak false
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Number) return (compareTo(obj) == 0);
		return super.equals(obj);
	}

	/**
	 * Funkce pevd slo ve form eskch slov na slo
	 * @param words slo ve form eskch slov
	 * @return slo
	 * @throws NumberFormatException
	 */
	public static int parseInt(String words) throws NumberFormatException {
		final String[] wordsTable  = {"jedn", "jede", "dve", "tri", "ctyri", "pet", "sest", "sedm", "sedum", "osm", "osum", "devet", "deset", "dva", "ctr", "pat", "deva", "pa", "sed", "nact", "cet", "desat", "esat", "sto", "ste", "sta", "set", "tisic", "milion", "miliard"};
		final char[]   actionTable = {    '+', 	  '+',   '+',   '+',     '+',   '+',    '+',    '+',     '+',   '+',    '+',     '+',     '+',   '+',   '+',  '+',     '+',  '+',  '+',    ',',   '*',     '*',     '*',   '*',   '*',   '*',   '*',     '^',     '^',       '^'};
		final int[]    numberTable = {      1,      1,     2,     3,       4,     5,      6,      7,       7,     8,      8,       9,      10,     2,     4,    5,       9,    5,    6,     10,    10,      10,      10,   100,   100,   100,   100,    1000, 1000000,1000000000};

		StringBuffer in = new StringBuffer(words.trim());

		int prevNumber = 0;
		int number = 0;
		int number2 = 0;

		int wordPos = 0;

		// projedeme cely vstupni text
		while(in.length() > 0) {
			// projedeme vsechny casti slov ve wordsTable
			while(wordsTable.length > wordPos) {
				// pokud je slovo z wordsTable nalezeno ve vstupnim textu
				if (in.length() >= wordsTable[wordPos].length() && in.substring(0, wordsTable[wordPos].length()).toString().compareToIgnoreCase(wordsTable[wordPos]) == 0) {
					//System.out.println(actionTable[wordPos] + wordsTable[wordPos]);
					// podle skupiny, do ktere slovo patri provedene prislusnou akci
					switch(actionTable[wordPos]) {
						case '+' :
							// skupina scitaci, provede zapamatovak
							if (prevNumber > 0) number+= prevNumber;
							prevNumber = numberTable[wordPos];
							break;
						case ',' :
							// skupina nactina, pricteme hodnotu deset
							number+= prevNumber + numberTable[wordPos];
							prevNumber = 0;
							break;
						case '*' :
							// skupina nasobna, provedem prislusne nasobeni
							if (prevNumber == 0) prevNumber = 1;
							number+= prevNumber * numberTable[wordPos];
							prevNumber = 0;
							break;
						case '^' :
							// skupina nasobna opakovaci, provedem nasobeni tisicem, milione, miliardou
							if (prevNumber > 0) {
								number+=prevNumber;
								prevNumber = 0;
							}
							if (number == 0) number = 1;
							// pokud jsme mimo rozsah vyhodime vyjimku
							if (numberTable[wordPos] == 1000000000 && number > 1) throw new NumberFormatException("Cislo je mimo rozsah.");
							number2+= number * numberTable[wordPos];
							number = 0;
							break;
					}
					// nakonec nalezene slovo vymazeme ze vstupniho textu
					in.delete(0, wordsTable[wordPos].length());
					break;
				}
				// nenalezli posuneme se na dalsi slovo ve wordsTable, pokud jsme ho nenalezli
				wordPos++;
			}
			// pokud nebyla nalezena vubez zadna shoda, vymazeme ze zacatku jedno pismeno a zkusime to znovu
			if (wordPos >= wordsTable.length) if (in.length() > 0) in.delete(0, 1);
			wordPos = 0;
		}
		// osetreni spravneho pricteni
		if (prevNumber > 0) number+= prevNumber;
		if (number > 0) number2+= number;

		return number2;

	}

	/**
	 * Funkce pevd slo na slo reprezentovan eskmi slovy
	 * @param number slo
	 * @return slo reprezentovan eskmi slovy
	 * @throws NumberFormatException
	 */
	public static String toString(int number) throws NumberFormatException {
		return toString(number, false);
	}

	/**
	 * Funkce pevd slo na slo reprezentovan eskmi slovy
	 * @param number slo
	 * @param useChequeFormat pout formt vstupu na sloenky
	 * @return slo reprezentovan eskmi slovy
	 * @throws NumberFormatException
	 */
	public static String toString(int number, boolean useChequeFormat) throws NumberFormatException {
		String[][] wordsTable = {
								 {"jedna", "dva", "tri", "ctyri", "pet", "sest", "sedm", "osm", "devet"},
								 {"jedenact", "dvanact", "trinact", "ctrnact", "patnact", "sestnact", "sedmnact", "osmnact", "devatenact"},
								 {"deset", "dvacet", "tricet", "ctyricet", "padesat", "sedesat", "sedmdesat", "osmdesat", "devadesat"},
								 {"sto", "dveste", "trista", "ctyrista", "petset", "sestset", "sedmset", "osmset", "devetset"},
								 {"tisic", "tisice", "tisic"},
								 {"milion", "miliony", "milionu"},
								 {"miliarda", "miliardy", "miliard"}
								};
		int digit = 0;
		String ret = "";

		// pokud je pouzit vypis na slozenky, pak provedeme par uprav, proto take wordsTable nema modifikator final
		if (useChequeFormat) {
			wordsTable[3][0] = "jednosto";
			wordsTable[4][0] = "jedentisic";
			wordsTable[5][0] = "jedenmilion";
			wordsTable[6][0] = "jednamiliarda";
		}

		// pokud je vstupni cislo mimo rozsah vyhodime vyjimku
		if (number > Math.pow(10, 9) || number < 0) throw new NumberFormatException("Vstupni cislo je mimo rozsah.");

		// pokud je cislo vetsi nez tisic
		if (number >= 1000) {
			// provedeme rekurzi, s tim ze pridame na konec vystupniho textu prislusne slovo (miliarda, milion, tisic)
			for(int exp = 3; exp >= 1; exp--) {
				if (number >= Math.pow(1000, exp)) {
					// zjistime, kolik je tam tech miliard, milionu, tisicu
					digit = number / (int) Math.pow(1000, exp);
					number-= digit * (int) Math.pow(1000, exp);

					if (digit > 1) {
						// rekurze, jeli digit vetsi nez jedna
						ret+= toString(digit, useChequeFormat);
						if (!useChequeFormat) ret+= " ";
					}

					// nakonec pridame slovo miliarda, milion, tisic spravne vysklonovane
					if (digit == 1) ret+= wordsTable[3 + exp][0];
					else if (digit < 5) ret+= wordsTable[3 + exp][1];
					else ret+= wordsTable[3 + exp][2];

					if (!useChequeFormat) ret+= " ";
				}
			}
		}
		ret = ret.trim();

		// nyni projedeme cisla nissi nez 1000
		if (number > 0) {
			// projedeme stovky, zjistime, kolik jich tam je
			digit = number / 100;
			if (digit > 0) {
				number-= digit * 100;
				if (!useChequeFormat) ret+= " ";
				ret+= wordsTable[3][digit - 1];
			}

			// projedeme desitky, zjistime, kolik jich tam je, jen dame pozor na cisla od 11 do 19, ty vyhodnocujeme jako nactine
			digit = number / 10;
			if (digit > 1 || number == 10) {
				number-= digit * 10;
				if (!useChequeFormat) ret+= " ";
				ret+= wordsTable[2][digit - 1];
			}
			// projedeme ...nact, zjistime kolik jich tam je
			digit = number - 10;
			if (number > 10 && number < 20) {
				number-= number;
				if (!useChequeFormat) ret+= " ";
				ret+= wordsTable[1][digit - 1];
			}
			// projedeme jednotky, zjistime kolik jich tam je
			if (number > 0) {
				if (!useChequeFormat) ret+= " ";
				ret+= wordsTable[0][number - 1];
			}
		}

		// vratime vystup
		return ret.trim();
	}
}
