package uir;
import uir.RomanFormatException;

/**
 * Tda implementuje pevody mezi slem a arabskmi literly<br>
 * <br>
 * Poznmka: Pracuje pouze s rozsahem od 0 do 3999.
 * @author Martin Sloup, A04372<br>msloup@students.zcu.cz<br><br>Zpadoesk Univerzita<br>Fakulta Aplikovanch Vd
 */
public class RomanNumber extends Number implements Comparable {
	private static final long serialVersionUID = 4092187208343552128L;
	private String stringValue;
	private int intValue;

	/**
	 * Konstruktor tdy s pouitm parametru reprezentovanm mskmi literly
	 * @param s 		hodnota reprezentovan mskmi literly
	 * @throws RomanFormatException
	 */
	public RomanNumber(String s) throws RomanFormatException {
		stringValue = s;
		intValue = parseInt(s);
	}

	/**
	 * Konstruktor tdy s pouitm parametru reprezentovanou v destkov soustav
	 * @param value		hodnota v destkov soustav
	 * @throws RomanFormatException
	 */
	public RomanNumber(int value) throws RomanFormatException {
		intValue = value;
		stringValue = toString(value);
	}

	/**
	 * Vrac slo reprezentovan mskmi literly
	 * @return slo reprezentovan mskmi literly
	 */
	public String toString() {
		return stringValue;
	}

	/**
	 * Vrac hodnotu typu int
	 * @return hodnota typu int
	 */
	public int intValue() {
		return intValue;
	}

	/**
	 * Vrac hodnotu typu short
	 * @return hodnota typu short
	 */
	public short shortValue() {
		return (short) intValue;
	}

	/**
	 * Vrac hodnotu typu long
	 * @return hodnota typu long
	 */
	public long longValue() {
		return (long) intValue;
	}

	/**
	 * Vrac hodnotu typu double
	 * @return hodnota typu double
	 */
	public double doubleValue() {
		return (double) intValue;
	}

	/**
	 * Vrac hodnotu typu float
	 * @return hodnota typu float
	 */
	public float floatValue() {
		return (float) intValue;
	}

	/**
	 * Vrac hodnotu typu byte
	 * @return hodnota typu byte
	 */
	public byte byteValue() {
		return (byte) intValue;
	}

	/**
	 * Porovnv s objektem podtypu typu Number
	 * @param arg0 objekt podtypu typu Number
	 * @return -1, pokud je arg0 vt; 0, pokud jsou stejn velk; 1, pokud je arg0 men
	 * @throws ClassCastException
	 */
	public int compareTo(Object arg0) throws ClassCastException {
		if ((arg0 instanceof Double) || (arg0 instanceof Float)) {
			double value = ((Number)arg0).doubleValue();
			if (this.doubleValue() == value) {
				return 0;
			} else {
				return (this.doubleValue() > value) ? 1 : -1;
			}
		}
		if (arg0 instanceof Number)
			return (int) (this.longValue() - ((Number)arg0).longValue());


		throw new ClassCastException("Objekt musi podtyp typu Number.");
	}

	/**
	 * Porovnv hodnotu objektu s hodnotou typu int
	 * @param arg0 hodnota typu int, kter bude porovnna
	 * @return -1, pokud je arg0 vt; 0, pokud jsou stejn velk; 1, pokud je arg0 men
	 */
	public int compareTo(int arg0) {
		return this.intValue() - ((Number)arg0).intValue();
	}

	/**
	 * Porovnva zda zadan objekt obsahuje stejnou hodnotu (podtyp typu Number), ppadn zda je objekt toton
	 * @param obj objekt kter bude porovnvn
	 * @return true, pokud maj stejn ou hodnotu, ppadn jsou toton, jinak false
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Number) return (compareTo(obj) == 0);
		return super.equals(obj);
	}

	/**
	 * Pevd arabsk sla (klasick slice) na mskou reprezentaci.
	 * @param arabic vstupn slo v rozsahu 0-3999
	 * @return arabsk reprezentace vstupnho arabskeho sla
	 * @throws RomanFormatException
	 */
	public static String toString(int arabic) throws RomanFormatException {
		//                                1,  10, 100,     5,  50, 500,    10, 100, 1000
		final String[][] romanTable = {{"I", "X", "C"}, {"V", "L", "D"}, {"X", "C", "M"}};
		String roman = "";
		int digit;

		//Osetrime meze
		if (arabic > 3999 || arabic < 0) throw new RomanFormatException("Cislo je mimo rozsah.");

		//Nula vraci prazdny retezec
		if (arabic == 0) return "";

		//nejprve projedeme tisice, zda je obsahuje
		digit = arabic / 1000;
		arabic-= digit * 1000;

		//zapiseme M tolikkrat, kolikrat je tam tisicu
		while(digit > 0) {
			roman+= "M";
			digit--;
		}

		//projedeme zbyle tri cislice
		for(int pos = 2; pos >= 0; pos--) {
			//zapamatujeme si jake je na prislusne pozici cislo
			digit = arabic / (int) Math.pow(10., (double) pos);
			arabic-= digit * (int) Math.pow(10., (double) pos);

			//pokud je to devitka
			if (digit==9) {
				//zapiseme I, X nebo C a nasleduje o jednotku vetsi cislo, tzn X, C, M
				roman+= romanTable[0][pos] + romanTable[2][pos];
			//pokud je to ctyrka
			} else if (digit==4) {
				//zapiseme I, X nebo C a nasleduje o jednotku vetsi cislo, tzn V, L, D
				roman+= romanTable[0][pos] + romanTable[1][pos];
			} else {
				// neni-li to 4 nebo 9, a je to vesi nez 4
				if (digit > 4) {
					//zapiseme V, L nebo C
					roman+= romanTable[1][pos];
					digit-= 5;
				}

				// a pak prislusny zbytek tolikrat, kolikarat je to potreba
				while(digit > 0) {
					//zapiseme I, X nebo C
					roman+= romanTable[0][pos];
					digit--;
				}
			}
		}

		return roman;
	}

	/**
	 * Pevd msk slice na arabskou reprezentaci
	 * @param roman	msk slice maximlne do sla MMMCMXCIX, przdn etzec odpovd 0
	 * @return slo v arabsk reprezentaci
	 * @throws RomanFormatException
	 */
	public static int parseInt(String roman) throws RomanFormatException {
		final char[] romanTable = {'I', 'V', 'X', 'L', 'C', 'D', 'M'};
		final int[] arabicTable = {  1,   5,  10,  50, 100, 500,1000};
		int arabic = 0;
		int prevNumber = 0;
		boolean found;

		roman = roman.trim();

		// projedeme cely vstupni retezec
		for(int pos = roman.length() - 1; pos >=0; pos--) {
			found = false;
			// heledame, zda znak v retezci je obsazen v tabulce
			for(int letter = 0; letter < romanTable.length; letter++) {
				if (Character.toUpperCase(roman.charAt(pos)) == romanTable[letter]) {
					found = true;
					// pokud predchozi znak reprezentoval vyssi cislo
					if (prevNumber > arabicTable[letter]) {
						// provadime odecet
						arabic-= arabicTable[letter];
					} else {
						// jinak scitame
						arabic+= arabicTable[letter];
						prevNumber = arabicTable[letter];
					}
				}
			}
			// pokud sme znak v tabulce nenalezli, hodime vyjimku o nespravnem formatu vstupu
			if (!found) throw new RomanFormatException("Nezname rimske literaly pro vstupni text: " + roman);
		}

		return arabic;
	}

	/**
	 * Funkce vrac true, pokud vstupn etzec obsahuje pouze msk literly
	 * @param roman vstupn etzec
	 * @return true, pokud vstupn etzec obsahuje pouze msk literly
	 */
	public static boolean isRoman(String roman) {
		final char[] romanTable = {'I', 'V', 'X', 'L', 'C', 'D', 'M'};
		boolean found;

		for(int pos = roman.length() - 1; pos >=0; pos--) {
			found = false;
			for(int letter = 0; letter < romanTable.length; letter++) {
				if (Character.toUpperCase(roman.charAt(pos)) == romanTable[letter]) {
					found = true;
					break;
				}
			}
			if (!found) return false;
		}
		return true;
	}
}
