import java.io.BufferedReader;
import java.io.InputStreamReader;

import uir.CzechWordsNumber;
import uir.RomanNumber;


/**
 * Tda hlavnho programu
 * @author sloup
 */
public class Main {
	private static char from = ' ';
	private static char to = ' ';
	private static String text = null;
	private static boolean useChequeFormat = false;

	/**
	 * Hlavni program
	 * @param args argumenty programu
	 */
	public static void main(String[] args) {
		parseArgs(args);

		if  (text != null) {
			System.out.println(translate(text, from, to));
		} else {
			try {
				BufferedReader inStream = new BufferedReader(new InputStreamReader(System.in));

				String inLine;

				while ((inLine = inStream.readLine()) != null) {
					System.out.println(translate(inLine, from, to));
				}
			} catch (Exception e) {
				System.err.println("Chyba: " + e.getMessage());
			}
		}
	}

	/**
	 * Provd pevod mezi soustavami
	 * @param text vstupn hodnota zdrojov soustavy
	 * @param from zdrojov soustava
	 * @param to clov soustava
	 * @return hodnota v clov soustav
	 */
	private static String translate(String text, char from, char to) {
		int value;
		try {
			//transformace vstupu do int
			switch(Character.toUpperCase(from)) {
				case 'N':
					value = Integer.parseInt(text);
					break;
				case 'R':
					value = RomanNumber.parseInt(text);
					break;
				case 'W':
					value = CzechWordsNumber.parseInt(text);
					break;
				default:
					if (RomanNumber.isRoman(text)) {
						value = RomanNumber.parseInt(text);
					} else if (isInteger(text)) {
						value = Integer.parseInt(text);
					} else {
						value = CzechWordsNumber.parseInt(text);
					}
					break;
			}

			//provedeni vystupni transformace
			switch(Character.toUpperCase(to)) {
				case 'N':
					return Integer.toString(value);
				case 'R':
					return RomanNumber.toString(value);
				case 'W':
					return CzechWordsNumber.toString(value, useChequeFormat);
			}
		} catch(Exception e) {
			System.err.println("Chyba: " + e.getMessage());
		}
		return "";
	}

	/**
	 * Program provd testovn, zda zadan text obsahuje slo
	 * @param text vstupn text
	 * @return true, pokud vstupn text je slo
	 */
	private static boolean isInteger(String text) {
		try {
			Integer.parseInt(text);
			return true;
		} catch(Exception e) {
			return false;
		}
	}

	/**
	 * Metoda provdjc parsovn vstupnch parametr programu
	 * @param args argumenty programu
	 */
	private static void parseArgs(String[] args) {
		StringBuilder sb;
		if (args.length >= 1) {
			char[] param = args[0].toUpperCase().toCharArray();

			//postupne nacitame vsechny parametry prvniho argumentu
			for (int i = 0; i < param.length; i++) {
				switch(param[i]) {
					case '-' :
					case '/' :
						break;
					case '?' :
					case 'H' :
						showHelp();
						System.exit(0);
						break;
					case 'B' :
						useChequeFormat = true;
						break;
					case 'N' :
					case 'R' :
					case 'W' :
						if (from == ' ') {
							from = param[i];
						} else {
							to = param[i];
						}
						break;
					default :
						break;
				}
			}
			// pokud vstupni text je obsazen ve vice castech prikazove radky, spojime ho do jednoho textu
			if (args.length > 1) {
				sb = new StringBuilder();
				for(int i = 1; i < args.length; i++) {
					if (sb.length() > 0) sb.append(' ');
					sb.append(args[i]);
				}
				text = sb.toString();
			}
			//opravime vyber vystupni transformace, pokud nebyla zadana vstupni
			if (to == ' ') {
				to = from;
				from = ' ';
			}
		}
		//pokud neni yadano vystupni transformace, zobrazime napovedu a skoncime
		if (to == ' ') {
			showHelp();
			System.exit(0);
		}
	}

	/**
	 * Funkce zobrazujc npovdu programu
	 */
	private static void showHelp() {
		System.out.println("KONVERTOR verze 1.0");
		System.out.println("Program provadi vzajemnou konverzi mezi cislem, rimskymi literaly a cislem");
		System.out.println("vyjadrenym pomoci ceskych slov.");
		System.out.println("Autor: Martin Arcao Sloup, A04372, KIV ZCU");
		System.out.println("Email: msloup@students.zcu.cz, arcao@arcao.com");
		System.out.println();
		System.out.println("POUZITI:");
		System.out.println("  java -jar konvertor.jar [zdroj][cil][atributy] [text]");
		System.out.println();
		System.out.println("Parametry ZDROJ, CIL:");
		System.out.println("  n    prevod z cisla / na cislo");
		System.out.println("  r    prevod z rimskych literalu / na rimske literaly");
		System.out.println("  w    prevod z ceskych slov / na ceska slova");
		System.out.println();
		System.out.println("Parametr ATRIBUTY:");
		System.out.println("  b    pouziti pravidel vypisu na slozenky pri prevodu na ceska slova");
		System.out.println("  h,?  zobrazeni teto napovedy");
		System.out.println();
		System.out.println("Poznamka:");
		System.out.println("  Pokud nebude uveden zdroj, program se pokusi o autodetekci vstupniho textu.");
		System.out.println("  Pokud nebude uveden vstupni text, program se bude snazit nacitat ze");
		System.out.println("  standardniho vstupu.");
		System.out.println();
		System.out.println("  Program rozeznava pouze ceska slova bez diakritiky, to z duvodu, ze neni");
		System.out.println("  mozne zarucit spravnost diakritiky pri vstupu z prikazove radky, pripadne");
		System.out.println("  jeho vystupu.");
	}


}
